﻿using System;
using System.IO;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using simplehttpserver;

namespace SimpleHttpServerTestProject
{
    [TestClass]
    public class HttpServerTest
    {
        private const string CrLf = "\r\n";
        private static HttpServer _server;

        [ClassInitialize]
        public static void Startup(TestContext context)
        {
            _server = new HttpServer(HttpServer.DefaultPort);
            Task.Run(() => _server.RunServer());
        }

        [ClassCleanup]
        public static void Shutdow()
        {
            _server.Stop();
        }

        [TestMethod]
        public void TestGet()
        {
            String line = GetFirstLine("GET /file.txt HTTP/1.0");
            Assert.AreEqual("HTTP/1.0 200 OK", line);

            line = GetFirstLine("GET /fileDoesNotExist.txt HTTP/1.0");
            Assert.AreEqual("HTTP/1.0 404 Not Found", line);
        }

        [TestMethod]
        public void TestGetIllegalRequest()
        {
            String line = GetFirstLine("GET /file.txt HTTP 1.0");
            Assert.AreEqual("HTTP/1.0 400 Bad Request", line);
        }

        [TestMethod]
        public void TestGetIllegalMethodName()
        {
            String line = GetFirstLine("PLET /file.txt HTTP/1.0");
            Assert.AreEqual("HTTP/1.0 400 Bad Request", line);
        }

        [TestMethod]
        public void TestGetIllegalProtocol()
        {
            String line = GetFirstLine("GET /file.txt HTTP/1.2");
            Assert.AreEqual("HTTP/1.0 400 Illegal protocol", line);
        }

        [TestMethod]
        public void TestMethodNotImplemented()
        {
            String line = GetFirstLine("POST /file.txt HTTP/1.0");
            Assert.AreEqual("HTTP/1.0 501 Not Implemented", line);
        }

        /// <summary>
        /// Private helper method
        /// </summary>
        /// <param name="request">The request to send to the server</param>
        /// <returns>The first line of the response</returns>
        private static String GetFirstLine(String request)
        {
            using (TcpClient client = new TcpClient("localhost", HttpServer.DefaultPort))
            using (NetworkStream networkStream = client.GetStream())
            using (StreamWriter toServer = new StreamWriter(networkStream, Encoding.UTF8))
            {
                toServer.Write(request + CrLf);
                toServer.Write(CrLf);
                toServer.Flush();

                StreamReader fromServer = new StreamReader(networkStream);
                String firstline = fromServer.ReadLine();
                toServer.Close();
                fromServer.Close();
                return firstline;
            }
        }
    }
}
